package com.dkha.server.controllers;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dkha.common.enums.YNEnums;
import com.dkha.common.exception.DkException;
import com.dkha.common.modules.vo.face.ApiSearchFaceRectVO;
import com.dkha.common.modules.vo.facelib.FileImageVo;
import com.dkha.common.modules.vo.position.PortraitListVo;
import com.dkha.common.page.PageParam;
import com.dkha.common.result.CommonResult;
import com.dkha.common.systemcode.SystemCode;
import com.dkha.common.util.IdCardUtil;
import com.dkha.common.util.UtilValidate;
import com.dkha.server.annotation.NoRepeatSubmitAnnotation;
import com.dkha.server.modules.entities.FaceLibrary;
import com.dkha.server.modules.entities.Portrait;
import com.dkha.server.services.IFaceLibraryService;
import com.dkha.server.services.IPortraitService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @version V1.0
 * @Description: TODO(please write your description)
 * All rights 成都电科慧安
 * @Title: ResourcesController
 * @Package com.dkha.server.controllers
 * @author: yangjun
 * @date: 2019/12/9 15:49
 * @Copyright: 成都电科慧安
 */
@RestController
@RequestMapping("/Resources")
@Api(tags="资源管理")
public class ResourcesController  extends CommonResult {
    public static final Logger logger = LoggerFactory.getLogger(ResourcesController.class);
    /**
     * 库
     */
    @Autowired
    private IFaceLibraryService iFaceLibraryService;
    /**
     * 人像
     */
    @Autowired
    private IPortraitService iPortraitService;
    @PostMapping("/faceLib")
    @ApiParam(required = true, name = "faceLibrary", value = "")
    @ApiOperation(value = "新增库")
    @RequiresPermissions("portrait:lib:save")
    @NoRepeatSubmitAnnotation(prefix = "Resources/faceLib")
    public CommonResult addFaceLibrary(@ApiParam(required = true, name = "faceLibrary", value = "{\n" +
            "  \"factoryName\": \"库名称\",\n" +
            "  \"remarks\": \"备注\"\n" +
            "}")@RequestBody FaceLibrary faceLibrary){
        if (UtilValidate.isEmpty(faceLibrary.getFactoryName())){
            throw new DkException("库名称不能为空");
        }
        faceLibrary.setIdFactory(System.currentTimeMillis());
        faceLibrary.setFactoryType("0");
        faceLibrary.setIsValid(YNEnums.YES.code);
        faceLibrary.setCreateTime(new Date());
        faceLibrary.setUpdateTime(new Date());
        FaceLibrary faceLibrar = iFaceLibraryService.addLibrary(faceLibrary);
        if(UtilValidate.isEmpty(faceLibrar))
        {
            return failResult("添加任性库失败失败");
        }
        return successResult(faceLibrar);
    }
    @PutMapping("/updateLibrary")
    @ApiParam(required = true, name = "faceLibrary", value = "")
    @ApiOperation(value = "修改库")
    @RequiresPermissions("portrait:lib:update")
    public CommonResult updateLibrary(@ApiParam(required = true, name = "faceLibrary", value = "{\n" +
            "  \"idFactory\": \"库id\",\n" +
            "  \"factoryName\": \"库名称\",\n" +
            "  \"factoryType\": \"库类型\",\n" +
            "  \"remarks\": \"备注\"\n" +
            "}")@RequestBody FaceLibrary faceLibrary){
        if (UtilValidate.isEmpty(faceLibrary.getIdFactory())){
            throw new DkException("库ID不能为空");
        }
        FaceLibrary faceLibrar = iFaceLibraryService.updateLibrary(faceLibrary);
        if(UtilValidate.isEmpty(faceLibrar))
        {
            return failResult("修改库失败");
        }
        return successResult(faceLibrar);
    }
    @DeleteMapping("/faceLib/{libraryId}")
    @ApiParam(required = true, name = "faceLibrary", value = "")
    @ApiOperation(value = "删除库")
    @RequiresPermissions("portrait:lib:del")
    public CommonResult deleteLibrary(@ApiParam(required = true, name = "libraryId", value = "")
                                      @PathVariable(value = "libraryId") Long libraryId){
        if (UtilValidate.isEmpty(libraryId)){
            throw new DkException("库id不能为空");
        }
        FaceLibrary faceLibrar;
        try {
            faceLibrar= iFaceLibraryService.deleteLibrary(libraryId);
        }catch (DkException e){
            logger.error(e.getMessage(), e);
            return failResult(Integer.valueOf(e.getCode()),e.getMsg(), null);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return failResult(SystemCode.HANDLER_FAILED.des);
        }
        return successResult(faceLibrar);
    }

    /**
     * 分页查询
     * @return
     */
    @PostMapping("/libraryPage")
    @ApiOperation(value = "人像库分页查询")
    @RequiresPermissions("portrait:lib:info")
    public CommonResult getLibraryPage(@ApiParam(required = true, name = "pageParam", value = "{\n" +
            "\t\"pageNo\": \"当前页码\",\n" +
            "\t\"pageSize\": \"每页显示数量 默认10\",\n" +
            "\t\"note\": {\n" +
            "\t\t\"factoryName\": \"库名称\"\n" +
            "\t}\n" +
            "}")@RequestBody PageParam pageParam){
        Page<FaceLibrary> page = iFaceLibraryService.queryLibrary(pageParam);
        return successResult(page);
    }
    @ApiOperation("上传人像附件")
    @PostMapping("/addFile/{libraryId}")
    @RequiresPermissions("portrait:save")
    //@NoRepeatSubmitAnnotation(prefix = "Resources/addFile")
    public CommonResult addFile(@PathVariable("libraryId") String libraryId, @RequestParam("processFile") MultipartFile multipartFile) {
        if(UtilValidate.isEmpty(libraryId) ){
            throw new DkException("目标库不能为空");
        }
        if(UtilValidate.isEmpty(multipartFile)){
            throw new DkException("请选择图片");
        }
        Portrait portrait;
        try {
            portrait = iPortraitService.addPortrait(libraryId, multipartFile);
        }catch (DkException e){
            logger.error(e.getMessage(), e);
            return failResult(Integer.valueOf(e.getCode()),e.getMsg(), null);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return failResult(SystemCode.HANDLER_FAILED.des);
        }

        return successResult(portrait);
    }

    @ApiOperation("布控图片上传人像附件")
    @PostMapping("/addpngFile/{libraryId}")
    @RequiresPermissions("portrait:save")
    public CommonResult addpngFile(@PathVariable("libraryId") String libraryId, @RequestParam("processFile") MultipartFile multipartFile) {
        if(UtilValidate.isEmpty(libraryId) ){
            throw new DkException("目标库不能为空");
        }
        if(UtilValidate.isEmpty(multipartFile)){
            throw new DkException("请选择图片");
        }
        Portrait portrait;
        try {
            portrait = iPortraitService.addPortraitPng(libraryId, multipartFile);
        }catch (DkException e){
            logger.error(e.getMessage(), e);
            return failResult(Integer.valueOf(e.getCode()),e.getMsg(), null);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return failResult(SystemCode.HANDLER_FAILED.des);
        }

        return successResult(portrait);
    }

    @ApiOperation("人像检测")
    @PostMapping("/pictureDetection")
    //@RequiresPermissions("employment:contract:upload")
    public CommonResult pictureDetection(@RequestParam("processFile") MultipartFile multipartFile) {

        if(UtilValidate.isEmpty(multipartFile)){
            throw new DkException("请选择图片");
        }
        List<ApiSearchFaceRectVO> apiSearchFaceRectVOS = new ArrayList<>();
        try{
            apiSearchFaceRectVOS = iPortraitService.pictureDetection(multipartFile);
        }catch (DkException e){
            logger.error(e.getMessage(), e);
            return failResult(Integer.valueOf(e.getCode()),e.getMsg(), null);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return failResult(SystemCode.HANDLER_FAILED.des);
        }
        return successResult(apiSearchFaceRectVOS);
    }
    @ApiOperation("批量上传人像Base64")
    @PostMapping("/batch/{libraryId}")
    //@RequiresPermissions("employment:contract:upload"
    public CommonResult addbatch(@PathVariable("libraryId") String libraryId, @RequestBody FileImageVo fileImageVo) {
        if(UtilValidate.isEmpty(libraryId) ){
            throw new DkException("目标库不能为空");
        }
        List<Portrait> portraits = new ArrayList<>();
        Portrait portrait = null;
        /**base64上传到文件服务器的地址*/
        if(UtilValidate.isEmpty(fileImageVo.getUrls())){
            throw new DkException("请选择图片");
        }else{
            try {
                if (fileImageVo.getUrls().size() > 0) {
                    for (String url :
                            fileImageVo.getUrls()) {
                        portrait = iPortraitService.addPortraitPngs(libraryId, url);
                        portraits.add(portrait);
                    }
                }
                } catch(Exception e){
                    logger.error("批量上传人像异常" + e.getMessage());
                    return failResult(e.getMessage());
                }
        }
        return successResult(portraits);
    }
    @ApiOperation("批量上传人像")
    @PostMapping("/addFiles/{libraryId}")
    //@RequiresPermissions("employment:contract:upload"
    public CommonResult addFiles(@PathVariable("libraryId") String libraryId, @RequestParam("processFile") MultipartFile[] multipartFile) {
        if(UtilValidate.isEmpty(libraryId) ){
            throw new DkException("目标库不能为空");
        }
        if(UtilValidate.isEmpty(multipartFile)){
            throw new DkException("请选择图片");
        }
        List<Portrait> portraits;
        try {
            portraits = iPortraitService.addPortraits(libraryId, multipartFile);
        } catch (Exception e) {
            logger.error("批量上传人像异常"+e.getMessage());
            return failResult(e.getMessage());
        }
        return successResult(portraits);
    }

    /**
     * 人脸信息分页查询
     * @return
     */
    @PostMapping("/queryPortrait")
    @ApiOperation(value = "人像分页查询")
    @RequiresPermissions("portrait:info")
    public CommonResult getPortraitVO(@ApiParam(required = true, name = "pageParam", value = "{\n" +
            "\t\"pageNo\": \"当前页码\",\n" +
            "\t\"pageSize\": \"每页显示数量 默认10\",\n" +
            "\t\"note\": {\n" +
            "\t\"name\": \"姓名\",\n" +
            "\t\"sex\": \"性别\",\n" +
            "\t\"libraryId\": \"库id\"\n" +
            "\t}\n" +
            "}")@RequestBody PageParam pageParam){
        //ReturnVO page = iPortraitService.queryPortrait(pageParam)
        Page<Portrait> page = iPortraitService.queryPortrait(pageParam);
        return successResult(page);
    }

    @PostMapping("/deletePortraits")
    @ApiParam(required = true, name = "idPortrait", value = "")
    @ApiOperation(value = "批量删除人像信息")
    @RequiresPermissions("portrait:del")
    public CommonResult deletePortraits(@ApiParam(required = true, name = "idPortrait", value = "")
                                        @RequestBody PortraitListVo idPortrait){
        if (UtilValidate.isEmpty(idPortrait)){
            throw new DkException("人像id不能为空");
        }
        List<Long> longs = iPortraitService.deletePortraits(idPortrait.getIdPortrait());
        if(UtilValidate.isEmpty(longs))
        {
            return failResult("删除人像失败");
        }
        return successResult(longs);
    }

    @DeleteMapping("/deletePortrait/{idPortrait}")
    @ApiParam(required = true, name = "idPortrait", value = "")
    @ApiOperation(value = "删除人像信息")
    @RequiresPermissions("portrait:del")
    public CommonResult deletePortrait(@ApiParam(required = true, name = "idPortrait", value = "")
                                       @PathVariable(value = "idPortrait") Long idPortrait){
        if (UtilValidate.isEmpty(idPortrait)){
            throw new DkException("人像id不能为空");
        }
        Long aLong = iPortraitService.deletePortrait(idPortrait);
        if(UtilValidate.isEmpty(aLong))
        {
            return failResult("删除人像失败");
        }
        return successResult(aLong);
    }

    @PutMapping("/updatePortrait")
    @ApiParam(required = true, name = "portrait", value = "")
    @ApiOperation(value = "修改人像信息")
    @RequiresPermissions("portrait:update")
    public CommonResult updatePortrait(@ApiParam(required = true, name = "portrait", value = "{\n" +
            "  \"idPortrait\": \"id\",\n" +
            "  \"idCard\": \"身份证\",\n" +
            "  \"name\": \"姓名\",\n" +
            "  \"sex\": \"性别\"\n" +
            "}") @RequestBody Portrait portrait){
        if (UtilValidate.isEmpty(portrait.getIdPortrait())){
            throw new DkException("id不能为空");
        }
        if (UtilValidate.isNotEmpty(portrait.getIdCard())) {
            if (!IdCardUtil.isValidatedAllIdcard(portrait.getIdCard())) {
                throw new DkException(SystemCode.IDCARD_ERROR.code,"身份证号码输入不正确");
            }
        }
        Portrait portrait1 = iPortraitService.updatePortrait(portrait);
        if(UtilValidate.isEmpty(portrait1))
        {
            return failResult("修改人像失败");
        }
        return successResult(portrait1);
    }
    @GetMapping(value = "/exportProjectTemplate")
    @ApiOperation(value = "导出项目管理excel模板", httpMethod = "GET")
    public void exportTemplate(HttpServletResponse response) {

        try {
            ClassPathResource classPathResource = new ClassPathResource("项目信息导入模板.xlsx");
            InputStream inputStream = classPathResource.getInputStream();
            response.setCharacterEncoding("UTF-8");
            response.setContentType("application/vnd.ms-excel;charset=UTF-8");
            response.setHeader("Content-Disposition",
                    "attachment;filename=" + URLEncoder.encode("项目信息导入模板", "UTF-8") + ".xlsx");
            OutputStream out = response.getOutputStream();
            int length = 0;
            byte[] bytes = new byte[1024];
            while ((length = inputStream.read(bytes)) != -1) {
                out.write(bytes, 0, length);
            }
            out.flush();
            out.close();
            inputStream.close();

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}